package org.jicengine;
import org.jicengine.io.Resource;
import java.util.Map;

/**
 *
 *
 *
 *
 * @author .timo
 *
 */

public class JICE {

	private static Builder INSTANCE = new org.jicengine.builder.BuilderImpl();

	public static Builder getBuilder()
	{
		return INSTANCE;
	}

	public static Object build(Instructions instructions) throws Exception
	{
		return getBuilder().build(instructions);
	}

	public static Object build(Resource jicFile, Map overridings) throws Exception
	{
		return build(new Instructions(jicFile, overridings));
	}

	public static Object build(Resource jicFile) throws Exception
	{
		return build(jicFile, null);
	}

	/**
	 * <p>
	 * command-line interface for processing a jic-file.
	 * </p>
	 * <p>format:
	 * <code>java org.jicengine.JICE -jic [jic-file] [-param name1 value1] [-param name2 value2] ..</code>
	 * </p>
	 */
	public static void main(String[] args) throws Exception
	{
		if( args.length == 0 ){
			throw new IllegalArgumentException("Usage: java org.jicengine.JICE -jic jic-file [-param name1 value1] [-param name2 value2] ..");
		}

		String jicFile = null;
		Map parameters = new java.util.HashMap();


		for (int i = 0; i < args.length; i++) {
			String arg = args[i];

			if( arg.startsWith("-") ){
				// ok, this is what was expected
				String name = arg.substring(1);

				if( name.equals("jic") ){
					i++;
					jicFile = getArgument(args, i, "expected '-jic' to be followed by a path.");
				}
				else if( name.equals("param") ){
					i++;
					String paramName = getArgument(args, i, "expected '-param' to be followed by the param name.");
					i++;
					String paramValue = getArgument(args, i, "expected '-param " + paramName + " to be followed by the param value.");
					parameters.put(paramName, paramValue);
				}
				else {
					throw new IllegalArgumentException("unknown argument: " + name);
				}
			}
			else {
				throw new IllegalArgumentException("expected an argument-name starting with '-', got '" + arg + "'");
			}
		}

		if( jicFile == null ){
			throw new IllegalArgumentException("argument -jic [jic-file] not specified.");
		}

		long now = System.currentTimeMillis();
		Object result = build(new org.jicengine.io.FileResource(new java.io.File(jicFile)), parameters);
		long elapsed = System.currentTimeMillis() - now;
		System.out.println("[JICE]: '" + jicFile + "' processed in " + elapsed + " ms. result:");
		System.out.println("  " + result);
		if( result != null ){
			System.out.println("  [" + result.getClass().getName() + "]");
		}
	}

	private static String getArgument(String[] args, int index, String errorMessage)
	{
		try {
			return args[index];
		} catch (ArrayIndexOutOfBoundsException e){
			throw new IllegalArgumentException(errorMessage);
		}
	}
}
