package org.jicengine.builder;
import javax.xml.parsers.*;
import org.jicengine.*;
import java.io.File;
import org.jicengine.element.Element;
import org.jicengine.element.ElementException;
import org.jicengine.element.ValueElement;
import org.jicengine.element.ActionElement;
import org.jicengine.operation.Context;
import org.jicengine.operation.SimpleContext;
/**
 *
 *
 *
 *
 * @author .timo
 *
 */

public class BuilderImpl implements Builder {

	private static boolean log = false;
	//Boolean.valueOf(System.getProperty("org.jicengine.Builder.log", "false")).booleanValue();

	public BuilderImpl() {
	}

	public Object build(Instructions instructions) throws Exception {

		// parse
		long begin = System.currentTimeMillis();
		Element rootElement = parse(instructions);
		long parseTime = System.currentTimeMillis() - begin;

		// execute the elements and return the result.
		begin = System.currentTimeMillis();
		Object result = execute(rootElement, instructions);
		long execTime = System.currentTimeMillis() - begin;

		if( log ){
			System.out.println("[" + instructions.getFile() + "]:");
			System.out.println("  parse time : " + parseTime);
			System.out.println("  exec time  : " + execTime);
			System.out.println("  total time : " + (parseTime+execTime));
			System.out.println();
		}

		return result;
	}

	private Element parse(Instructions instructions) throws org.xml.sax.SAXException, Exception
	{
		SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setNamespaceAware(true);
		factory.setFeature("http://xml.org/sax/features/namespaces", true);
		factory.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
		SAXParser parser = factory.newSAXParser();

		Handler handler = new Handler();

		try {
			parser.parse(instructions.getFile().getInputStream(), handler, instructions.getFile().getIdentifier());
			return handler.getResult();
		} catch (org.xml.sax.SAXException e){
			if( e.getException() != null ){
				throw e.getException();
			}
			else {
				throw e;
			}
		}
	}

	private Object execute(Element rootElement, Instructions buildInstructions) throws ElementException
	{
		Context globalContext = new SimpleContext("document-contex");
		BuildContext jiceContext = new BuildContextImpl(buildInstructions);
		globalContext.addObject(BuildContext.VARIABLE_NAME, jiceContext);

		if( rootElement instanceof ValueElement ){
			return ((ValueElement) rootElement).getValue(globalContext, null);
		}
		else if(rootElement instanceof ActionElement ){

			((ActionElement) rootElement).execute(globalContext, null);
			return null;
		}
		else {
			throw new IllegalStateException("unknown element type: " + rootElement);
		}
	}

	public static void main(String[] args) throws Exception
	{
		if( args.length == 0 ){
			System.out.println("give the org.jicengine-file as an argument.");
		}
		else {
			File file = new File(args[0]);
			System.out.println("-------------------");
			System.out.println("Processing file '" + file + "'");
			System.out.println("-------------------");
			java.util.Map parameters = new java.util.HashMap();
			parameters.put("language.code", "fi");
			Object result = new BuilderImpl().build(new Instructions(new org.jicengine.io.FileResource(file),parameters));

			System.out.println("-------------------");
			System.out.println("Got object:");
			System.out.println("  " + result);
			System.out.println("  (" + result.getClass().getName() + ")");
			System.out.println("-------------------");
		}

		/*
		javax.swing.JFrame frame = new javax.swing.JFrame("Anne on taas kiva");
		frame.setLocation(300,300);
		frame.setSize(200,200);
		frame.show();
		*/
	}
}
