package org.jicengine.builder;

import org.jicengine.operation.Context;
import org.jicengine.operation.SimpleContext;
import org.jicengine.operation.BeanUtils;
import org.jicengine.operation.OperationException;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.util.*;
import org.jicengine.element.*;
import org.jicengine.element.impl.*;
import org.jicengine.expression.*;
import org.jicengine.util.Utils;
import org.jicengine.Instructions;
import org.jicengine.BuildContext;

/**
 *
 * @author    .timo
 * @created   13. marraskuuta 2003
 */

public class Handler extends DefaultHandler {

	public static final String NAMESPACE_URI_JICE_1_0 = "http://www.jicengine.org/jic/1.0";

	static final String ATTR_NAME_ACTION = "action";
	static final String ATTR_NAME_CLASS = "class";
	static final String ATTR_NAME_ID = "id";
	static final String ATTR_NAME_PARAMETERS = "parameters";
	static final String ATTR_NAME_TRACE = "trace";
	static final String ATTR_NAME_TYPE = "type";
	static final String ATTR_NAME_INSTANCE = "instance";
	static final String ATTR_NAME_OVERRIDABLE_BY = "overridable-by";
	static final String ATTR_NAME_ELEMENT_NAME = "name";

	private Stack elementCompilers = new Stack();
	private Stack elements = new Stack();
	private Element rootElement;

	private String jiceNamespacePrefix = "";
	private boolean defaultNamespaceUsed = true;

	StringBuffer cdataContent = new StringBuffer();
	Locator locator;

	public Handler()
	{
	}

	public Element getResult() throws ElementException, OperationException
	{
		if( this.rootElement == null ){
			throw new IllegalStateException("root element is null.");
		}

		return this.rootElement;
	}

	public void setDocumentLocator(Locator locator)
	{
		this.locator = locator;
	}

	public void startDocument() throws org.xml.sax.SAXException
	{
	}

	public void endDocument() throws org.xml.sax.SAXException
	{
	}

	public void startPrefixMapping(String prefix, String uri) throws org.xml.sax.SAXException
	{
		if( uri.equals(NAMESPACE_URI_JICE_1_0) ){
			// the JICE namespace prefix has been specified.
			this.jiceNamespacePrefix = prefix;
			if( this.jiceNamespacePrefix == null || this.jiceNamespacePrefix.length() == 0){
				this.defaultNamespaceUsed = true;
			}
			else {
				this.defaultNamespaceUsed = false;
			}
		}
	}

	public void endPrefixMapping(String prefix) throws org.xml.sax.SAXException
	{
		// should we do something if the current org.jicengine-namespace-prefix-mapping ends?
		// or are we informed with a startPrefixMapping()-call of the future
		// prefixes?
	}

	/**
	 * @param jiceAttributeName the local name of a JICE-attribute e.g. 'action'.
	 */
	protected String getAttribute(Attributes attributes, String jiceAttributeName)
	{
		if( this.defaultNamespaceUsed ){
			// JICE-namespace is bound to the default namespace.
			// the attributes don't actually belong to any namespace..
			// anyway, we can use the jiceAttributeName directly.
			String value = attributes.getValue(jiceAttributeName);
			return value;
		}
		else {
			// ok, lets specify the namespace also.
			return attributes.getValue(NAMESPACE_URI_JICE_1_0, jiceAttributeName);
		}
	}

	public void startElement(String nameSpaceUri, String localName, String qName, Attributes attributes) throws org.xml.sax.SAXException
	{
		try {
			doStartElement(nameSpaceUri,localName,qName,attributes);
		} catch (Exception e){
			throw new org.xml.sax.SAXException(e);
		}
	}

	protected void doStartElement(String nameSpaceUri, String localName, String qName, Attributes attributes) throws Exception, org.xml.sax.SAXException
	{
		Location location;
		if( this.locator != null ){
			location = new Location(this.locator.getLineNumber(), locator.getSystemId(), this.elementCompilers.size());
		}
		else {
			location = new Location(-1, "?", 0);
		}

		if( !NAMESPACE_URI_JICE_1_0.equals(nameSpaceUri) ){
			throw new ElementException("Element '" + localName +"' is in illegal namespace '" + nameSpaceUri + "'. only '" + NAMESPACE_URI_JICE_1_0 + "' is supported.", localName, location);
		}
		final ElementCompiler current;

		// possible feature: a 'name'-attribute, that specifies the name instead
		// of the local-name?
		String name = getAttribute(attributes, ATTR_NAME_ELEMENT_NAME);
		if( name == null ){
			// most elements won't use the 'name'-attribute
			name = localName;
		}



		String typeExpression = getAttribute(attributes, "type");
		if( typeExpression == null ){
			typeExpression = "bean";
		}


		Type type = Type.parse(typeExpression);

		ElementCompiler parentElementCompiler = null;

		if( !this.elementCompilers.isEmpty()){
			// not a root..
			parentElementCompiler = (ElementCompiler)this.elementCompilers.peek();
		}

		current = DefaultJiceTypes.getTypeManager().createCompiler(name, location, type);

		// ATTRIBUTE HANDLING

		String attrName;
		String attrValue;
		String attrUri;
		for (int i = 0; i < attributes.getLength(); i++) {

			attrName = attributes.getLocalName(i);
			attrValue = attributes.getValue(i);
			attrUri = attributes.getURI(i);

			if( attrName.length() == 0 ){
				// attribute 'xmlns'. ignore
				continue;
			}
			else if( attrUri.length() == 0 && this.defaultNamespaceUsed ){
				// JICE-namespace is mapped as default namespace.
				// the attributes don't actually have namespace now..
				// but we accept any attributes without a namespace now.
				setAttribute(attrName, attrValue, current);
			}
			else if( NAMESPACE_URI_JICE_1_0.equals(attrUri) ){
				// a clear case, this is a JICE-attribute.
				setAttribute(attrName, attrValue, current);
			}
			else {
				// this attribute has no namespace although JICE-attributes have.
				// what to do with attributes that are in another namespace?
			}
		}

		// tell the element that it's attributes have been set
		current.elementInitialized();

		this.elementCompilers.push(current);
	}

	protected void setAttribute(String name, String value, ElementCompiler elementCompiler) throws Exception
	{
		if( name.equals(ATTR_NAME_CLASS) ){
			elementCompiler.setInstanceClass(value);
		}
		else if( name.equals(ATTR_NAME_TYPE) ){
			// nothing..
		}
		else if( name.equals(ATTR_NAME_ELEMENT_NAME) ){
			// nothing..
		}
		else if( name.equals(ATTR_NAME_INSTANCE) ){
			elementCompiler.setConstructor(value);
		}
		else if( name.equals(ATTR_NAME_ACTION) ){
			elementCompiler.setAction(value);
		}
		else if( name.equals(ATTR_NAME_PARAMETERS) ){
			elementCompiler.setParameters(value);
		}
		else if( name.equals(ATTR_NAME_ID) ){
			elementCompiler.setId(value);
		}
		else if( name.equals(ATTR_NAME_OVERRIDABLE_BY) ){
			elementCompiler.setOverridableBy(value);
		}
		else {
			try {
				BeanUtils.setProperty(elementCompiler, name, value);
			} catch (Exception e){
				throw new ElementException("Unsupported attribute: " + name + "=" + value, e, elementCompiler.getName(), elementCompiler.getLocation());
			}
		}
	}

	public void endElement(String namespaceUri, String localName, String qName) throws org.xml.sax.SAXException
	{
		try {
			doEndElement(namespaceUri,localName, qName);
		} catch (ElementException e){
			throw new org.xml.sax.SAXException(e);
		} catch (RuntimeException e2){
			throw new org.xml.sax.SAXException(new org.jicengine.util.ChainedException("Unexpected runtime exception at line " + this.locator.getLineNumber(), e2));
		} catch (Exception e3) {
			throw new org.xml.sax.SAXException(e3);
		}
	}

	protected void doEndElement(String namespaceUri, String localName, String qName) throws Exception, org.xml.sax.SAXException
	{
		//System.out.println("end-element: " + localName + ", stack=" + this.elementCompilers.size());

		ElementCompiler current = (ElementCompiler) this.elementCompilers.pop();
		//System.out.println("</" + current.getName() + ">");

		// CDATA of the element
		//
		// NOTE: because we clear the buffer here, there can't be nested
		// cdata-blocks. Consider the following situation:
		// --------
		// <parent>
		//  some text as CData
		//  <child>text of the child-element</child>
		// </parent>
		// --------
		// in that case, the child will receive all the cdata,also
		// the cdata of the parent, and the parent won't receive any
		// cdata at all (the child has consumed it all).
		String cdata = cdataContent.toString().trim();
		if( cdata.length() > 0 ){
			current.setCData(cdata);
		}
		cdataContent = new StringBuffer();

		// element processed.. let the compiler create the element.

		Element element = current.createElement();

		if( !this.elementCompilers.isEmpty() ){
			ElementCompiler parent = (ElementCompiler) this.elementCompilers.peek();

			// let the parent handle the element
			//System.out.println("<" + parent.getName() + "> handles child <" + element.getName() + ">");
			parent.handleChildElement(element);
		}
		else {
			// no parents: this is the root element?
			this.rootElement = element;
		}

	}

	public void characters(char[] cdata, int start, int length) throws org.xml.sax.SAXException
	{
		this.cdataContent.append(cdata, start, length);
	}

	public void ignorableWhitespace(char[] parm1, int parm2, int parm3) throws org.xml.sax.SAXException
	{
	}

	public void processingInstruction(String parm1, String parm2) throws org.xml.sax.SAXException
	{
	}

	public void skippedEntity(String parm1) throws org.xml.sax.SAXException
	{
	}
}
