package org.jicengine.element.impl;

import org.jicengine.operation.OperationException;

import org.jicengine.operation.Context;

import org.jicengine.operation.Operation;

import org.jicengine.element.*;
import java.util.*;

/**
 * NOTE: currently, every value-element is handled as an array-element. what if
 * the element has an action that would need some of the child-value-elements? .
 *
 * @author    .timo
 * @created   5. huhtikuuta 2004
 */

public class ArrayElementCompiler extends StrictElementCompiler {

	// the constructor accepts all parameters and saves the names
	// of the queried parameter.
	/**
	 * Description of the Class
	 *
	 * @author    timo
	 * @created   27. elokuuta 2004
	 */
	public class ArrayConstructor implements Operation {
		public boolean needsParameters()
		{
			return true;
		}

		public boolean needsParameter(String name)
		{
			if( name.startsWith("this[") && name.endsWith("]") ){
				return true;
			}
			else {
				return false;
			}
		}

		public Object execute(Context context) throws OperationException
		{
			List arrayElements = new ArrayList();

			try {
				int index = 0;
				String elementName = "this[" + index + "]";
				while(context.hasObject(elementName)){
					Object element = context.getObject(elementName);
					arrayElements.add(element);
					index++;
					elementName = "this[" + index + "]";
				}
			} catch(Exception e) {
				throw new ElementException(e, getName(), getLocation());
			}

			Class arrayType = getElement().getInstanceClass();
			if( arrayType == null ){
				arrayType = Object[].class;
			}

			Object arrayObject = java.lang.reflect.Array.newInstance(arrayType.getComponentType(), arrayElements.size());

			for(int i = 0; i < arrayElements.size(); i++) {
				Object element = arrayElements.get(i);
				try {
					java.lang.reflect.Array.set(arrayObject, i, element);
				} catch (IllegalArgumentException e){
					throw new OperationException("[array-position " + i + "]: failed to store value '" + element + " (class '" + element.getClass().getName() + "')");
				}
			}
			return arrayObject;
		}
	}


	/**
	 * Description of the Class
	 *
	 * @author    timo
	 * @created   27. elokuuta 2004
	 */
	public static class RenamedValueElement implements ValueElement {
		private String name;
		private ValueElement element;

		public RenamedValueElement(String name, ValueElement element)
		{
			this.name = name;
			this.element = element;
		}

		public String getName()
		{
			return this.name;
		}

		public Object getValue(Context global, Object parentInstance) throws ElementException
		{
			return element.getValue(global, parentInstance);
		}
	}

	private List elements = new ArrayList();
	private Class elementClass;
	private int index = 0;

	public ArrayElementCompiler(String name, Location location)
	{
		super(name, location);

		getElement().setConstructor(new ArrayConstructor());

	}

	/**
	 * <p>
	 *
	 * Overrides the default handleChildElement()-procedure so that it becomes
	 * possible to handle the child-elements as elements of the array. </p> <p>
	 *
	 * every ValueElement becomes an element in the array. we have to do some
	 * tricks, though. <p>
	 *
	 * <p>
	 *
	 * We rename every value-element using syntax 'this[0]', 'this[1]',
	 * 'this[2]',... with this scheme, we accomplish 3 things:
	 * <ol>
	 *   <li> <b>the constructor can recognize the array elements.</b> - all array
	 *   elements are handled as constructor parameters. the constructor must fetch
	 *   each parameter from the context by their name. using this syntax, the
	 *   array-constructor can query the context for array-elements. </li>
	 *   <li> <b> the constructor can resolve the size of the array</b> - the
	 *   array-elements are numbered, so by querying the context the constructor
	 *   can resolve the index of the last element. </li>
	 *   <li> <b> duplicate-id collisions are avoided </b> - every array-element is
	 *   added to the constructor-context that doesn't allow duplicate names. every
	 *   name must be different.
	 * </ol>
	 * <7p> <p>
	 *
	 * the renaming is done with wrapper-element RenamedValueElement. </p>
	 *
	 * @param child              Description of the Parameter
	 * @throws ElementException  Description of the Exception
	 */
	public void handleChildElement(Element child) throws ElementException
	{
		Element preparedChild;
		if(child instanceof ValueElement) {
			// every value element is handled as an array element.
			// rename

			preparedChild = new RenamedValueElement("this[" + index + "]", (ValueElement)child);
			index++;
		}
		else {
			preparedChild = child;
		}

		// let the super to the rest..
		super.handleChildElement(preparedChild);
	}
}
