package org.jicengine.expression;

import org.jicengine.operation.SimpleMethodInvocationOperation;
import org.jicengine.operation.MethodInvocationOperation;
import org.jicengine.operation.ConstructionOperation;
import org.jicengine.operation.ArrayConstructionOperation;
import org.jicengine.operation.Operation;
import java.util.*;

/**
 *
 *
 *
 *
 * @author .timo
 */

public class MethodInvocationParser implements Parser {

	Parser elementParser;

	public MethodInvocationParser(Parser elementParser)
	{
		this.elementParser = elementParser;
	}

	public Operation parse(String expression) throws SyntaxException
	{
		if( expression.startsWith("new ") ){
			// a 'new' operation, either:
			// a) 'new full.class.name(parameters)'
			// b) 'new full.class.name[arraySize]'

			int parameterPartStart = expression.indexOf("(");

			if ( parameterPartStart != -1 ){
				// not an array
				// extract class-name, like 'java.util.Date' from expression 'new java.util.Date()'.
				String className = expression.substring(4,parameterPartStart);
				String parameters = extractParameters(expression);
				return new ConstructionOperation(expression, elementParser.parse(className), parseParameters(parameters));
			}
			else {
				int arrayLengthStart = expression.indexOf("[");

				if( arrayLengthStart != -1 ){
					// looks like an array creation..
					String componentType = expression.substring(4,arrayLengthStart);
					String length = expression.substring(arrayLengthStart+1, expression.lastIndexOf("]"));
					return new ArrayConstructionOperation(expression, elementParser.parse(componentType), elementParser.parse(length));
				}
				else {
					throw new SyntaxException("expected 'new full.class.name(parameters)' or 'new full.class.name[arraySize]', got '" + expression + "'");
				}
			}
		}
		else {
			// if the 'new' operator is not used, then some method is called, either a static
			// or a method of some instance.

			// extract the part before "()" marks.
			String beforeMethodParameters = expression.substring(0, expression.indexOf("("));

			// extract the actor-part, a name of a variable, a class or a new operation.
			String actorName = beforeMethodParameters.substring(0, beforeMethodParameters.lastIndexOf("."));
			String method = beforeMethodParameters.substring( beforeMethodParameters.lastIndexOf(".")+1);

			Operation actor = elementParser.parse(actorName);

			Operation[] parameters = parseParameters(extractParameters(expression));

			return new SimpleMethodInvocationOperation(expression, actor, method, parameters);
		}
	}

		private Operation[] parseParameters(String parameterEnumeration) throws SyntaxException {

			String[] params = Utils.parseParameterlist(parameterEnumeration);
			Operation[] paramsAsOperations = new Operation[params.length];
			for (int i = 0; i < params.length; i++) {
				paramsAsOperations[i] = elementParser.parse(params[i]);
			}

			return paramsAsOperations;
		}

		/**
		 * extracts parameters-enumeration from an expression.
		 * Like 'param1,param2' from expression new full.package.name.ClassName(param1,param2)'
		 */
		private static String extractParameters(String expression){
			try {
				return expression.substring(expression.indexOf("(")+1, expression.lastIndexOf(")"));
			} catch (StringIndexOutOfBoundsException e){
				throw new IllegalArgumentException("Expression '" + expression + "' doesn't have valid parameters-part enclosed inside ()");
			}
		}

}
