package org.jicengine.expression;

import org.jicengine.operation.StaticValue;

/**
 *
 *
 *
 *
 * @author .timo
 *
 */

public class NumberParser extends StaticValueParser {

	public static final NumberParser INSTANCE = new NumberParser();

	public final static char NEGATIVE_SIGN = '-';
	public final static char DECIMAL_SEPARATOR = '.';
	public final static String DECIMAL_SEPARATOR_STRING = new String(new char[]{DECIMAL_SEPARATOR});
	public final static char FLOAT_MARKER_1 = new String("f").charAt(0);
	public final static char FLOAT_MARKER_2 = new String("F").charAt(0);
	public final static char LONG_MARKER_1 = new String("l").charAt(0);
	public final static char LONG_MARKER_2 = new String("L").charAt(0);
	public final static char DOUBLE_MARKER_1 = new String("d").charAt(0);
	public final static char DOUBLE_MARKER_2 = new String("D").charAt(0);

	protected Object parseValue(String expression) throws SyntaxException
	{
		if( expression.length() == 0 ){
			throw new IllegalArgumentException("Empty expression can't be parsed.");
		}
		char first = expression.charAt(0);
		if( Character.isDigit(first) || (first == NEGATIVE_SIGN && expression.length() > 1 && Character.isDigit(expression.charAt(1))) ){
			// looks like a number..
			try {
				return toNumber(expression);
			} catch (NumberFormatException e){
				throw new SyntaxException("Failed to parse numeric expression '" + expression + "' into a number.", e);
			}

		}
		else {
			// not a number..
			return null;
		}
	}

	protected Number toNumber(String expression) throws NumberFormatException
	{
		Number number = null;
		boolean hasDecimals = (expression.indexOf(DECIMAL_SEPARATOR) != -1);

		char lastChar = expression.charAt(expression.length()-1);

		if( Character.isDigit(lastChar) ){
			// only numbers -> integer or double
			if( hasDecimals ){
				number = new Double(expression);
			}
			else {
				number = new Integer(expression);
			}
		}
		else {

			if( lastChar == FLOAT_MARKER_1 || lastChar == FLOAT_MARKER_2 ){
				// float
				number = new Float(expression.substring(0,expression.length()-1));
			}
			else if( lastChar == LONG_MARKER_1 || lastChar == LONG_MARKER_2 ){
				// long
				number = new Long(expression.substring(0,expression.length()-1));
			}
			else if( lastChar == DOUBLE_MARKER_1 || lastChar == DOUBLE_MARKER_2 ){
				// double
				//number = new Double(expression.substring(0,expression.length()-1));
				number = new Double(expression);
			}
			else {
				throw new NumberFormatException("Couldn't understand the last character in numeric expression '" + expression + "'.");
			}
		}
		return number;
	}
}
