package org.jicengine.expression;

import java.util.List;
import java.util.ArrayList;
/**
 *
 *
 *
 *
 * @author .timo
 *
 */
public final class Utils {
	public static char STRING_MARKER = new String("'").charAt(0);
	public static char METHOD_PARAMS_START = new String("(").charAt(0);
	public static char METHOD_PARAMS_END = new String(")").charAt(0);

	/**
	 *
	 */
	public static Object[] parseSignature(String expression)
	{
		char[] chars = expression.toCharArray();
		char character;
		int paramStart = -1;
		boolean inString = false;
		for (int i = 0; i < chars.length; i++) {
			character = chars[i];
			if( character == METHOD_PARAMS_START && !inString){
				paramStart = i;
				break;
			}
			else if( character == STRING_MARKER ){
				if( !inString ){
					inString = true;
					continue;
				}
				else {
					inString = false;
					continue;
				}
			}
		}

		String beforeParameters;
		String[] parameters;

		if( paramStart < 0 ){
			beforeParameters = expression;
			parameters = new String[0];
		}
		else {
		 beforeParameters = expression.substring(0,paramStart);
		 parameters = extractParameters(expression,paramStart,expression.length());
		}

		return new Object[]{beforeParameters,parameters};
	}

	/**
	 * 'anything.anything(something, else)' -> ['something','else'].
	 */
	public static String[] extractParameters(String expression)
	{
		return extractParameters(expression, 0, expression.length());
	}

	public static String[] extractParameters(String expression, int fromIndex, int endIndex)
	{
		char[] chars = expression.toCharArray();
		char character;
		int paramStart = -1;
		int paramEnd = -1;
		boolean inString = false;
		for (int i = fromIndex; i < endIndex; i++) {
			character = chars[i];
			if( character == METHOD_PARAMS_START && !inString){
				paramStart = i;
				break;
			}
			else if( character == STRING_MARKER ){
				if( !inString ){
					inString = true;
					continue;
				}
				else {
					inString = false;
					continue;
				}
			}
		}

		inString = false;
		for (int i = endIndex-1; fromIndex <= i ; i--) {
			character = chars[i];
			if( character == METHOD_PARAMS_END && !inString){
				paramEnd = i;
				break;
			}
			else if( character == STRING_MARKER ){
				if( !inString ){
					inString = true;
					continue;
				}
				else {
					inString = false;
					continue;
				}
			}
		}

		if( paramStart < 0 && paramEnd < 0 ){
			return new String[0];
		}
		else {
			try {
				return parseParameterlist(expression.substring(paramStart+1,paramEnd));
			} catch (StringIndexOutOfBoundsException e){
				throw new IllegalArgumentException("didn't find correct parameter-part from string '" + expression + "' at [" + fromIndex + ".." + endIndex + "]");
			}
		}
	}

	public static String toParameterList(String[] parameters)
	{
		String list = "";
		for (int i = 0; i < parameters.length; i++) {
			list += parameters[i];
			if( i+1 < parameters.length ){
				list += ",";
			}
		}
		return list;

	}

	public static String[] parseParameterlist(String list)
	{
		if( list.indexOf("'") == -1 ){
			java.util.StringTokenizer tokenizer = new java.util.StringTokenizer(list,",");
			String[] params = new String[tokenizer.countTokens()];
			for (int i = 0; i < params.length; i++) {
				params[i] = tokenizer.nextToken().trim();
			}
			return params;
		}
		else {
			boolean inString = false;
			java.util.StringTokenizer tokenizer = new java.util.StringTokenizer(list,",'",true);
			List params = new ArrayList();
			String token;
			while( tokenizer.hasMoreElements() ){
				token = tokenizer.nextToken(",'");
				if( token.equals("'") ){
					try {
						token = tokenizer.nextToken("'");
						tokenizer.nextToken("'");
						params.add("'" + token + "'");
					} catch (java.util.NoSuchElementException e){
						throw new IllegalArgumentException("unclosed string in parameter-list: " + list);
					}
				}
				else if( token.equals(",") ){
					// nothing..
				}
				else {
					params.add(token.trim());
				}
			}

			return (String[]) params.toArray(new String[0]);
		}


	}
}
