package org.jicengine.operation;

import org.jicengine.expression.*;
/**
 * Context : a namespace for objects.
 *
 * @author .timo
 */

public interface Context {

	/**
	 * finds an object from the context.
	 *
	 * @return  the object. NOTE: currently no null values supported.
	 * @throws ObjectNotFoundException  if the context doesn't have an object
	 * with the given name.
	 */
	public Object getObject(String name) throws ObjectNotFoundException;

	public boolean hasObject(String name);

	/**
	 * If addObject success, the object stored will be returned by getObject
	 * with the same name.
	 *
	 * @throws DuplicateNameException if there already is another object with the
	 * same name in the context.
	 */
	public void addObject(String name, Object object) throws DuplicateNameException;


	public static class ObjectNotFoundException extends java.lang.RuntimeException {
		public ObjectNotFoundException(String objectName,Context context)
		{
			super("Object '" + objectName + "' not found in context " + context);
		}
		public ObjectNotFoundException(String message)
		{
			super(message);
		}
	}

	public static class DuplicateNameException extends java.lang.RuntimeException {
		public DuplicateNameException(String name,Object old, Object newObject, Context context)
		{
			super("'" + name + "' already reserved to '" + old + "' (" + old.getClass().getName() + "), can't add '" + newObject + "' (" + newObject.getClass().getName() + ") into context " + context);
		}
	}
}
