package org.jicengine.operation;

import org.jicengine.operation.SimpleContext;

import org.jicengine.expression.*;
import org.jicengine.operation.AbstractContext;

import org.jicengine.operation.Context;
import java.util.Map;
import java.util.HashMap;

/**
 * <p>
 * local context is a context inside a larger context (parent context).
 * objects in the local context override objects of the parent context.
 * if an object is not found from the local context, the parent context
 * is searched.
 * </p>
 *
 * <p>
 * new objects are added only to the local context.
 * </p>
 * <p>
 * the local context is represented by another Local-instance. this way
 * its implementation can be changed.
 * </p>
 *
 * @author .timo
 */

public class LocalContext extends AbstractContext {

	Context local;
	Context parent;

	public LocalContext(Context parent)
	{
		this(new SimpleContext(), parent);
	}

	/**
	 *
	 */
	public LocalContext(Context local, Context parent)
	{
		super(getName(local,"local-context"));
		this.local = local;
		setParent(parent);
	}

	public LocalContext(String name, Context local, Context parent)
	{
		super("local:" + local + ", parent:" + parent);
		this.local = local;
		setParent(parent);
	}

	public void setParent(Context context)
	{
		this.parent = context;
	}

	public Context getParent()
	{
		return this.parent;
	}

	public Object getObject(String name)
	{
		if( this.local.hasObject(name) ){
			return this.local.getObject(name);
		}
		else {
			try {
				return getFromParent(name);
			} catch (Context.ObjectNotFoundException e){
				throw new Context.ObjectNotFoundException(name, this);
			}
		}
	}

	protected Object getFromParent(String name)
	{
		return getParent().getObject(name);
	}

	public boolean hasObject(String name) {
		return (this.local.hasObject(name)) || (hasInParent(name));
	}

	protected boolean hasInParent(String name)
	{
		return getParent().hasObject(name);
	}

	public void addObject(String name, Object object)
	{
		this.local.addObject(name,object);
	}

	public String toString()
	{
		return "" + this.local + " + " + this.parent;
	}
}
