package org.jicengine.operation;

import java.util.List;
import java.util.ArrayList;

/**
 * @author    .timo
 * @created   14. marraskuuta 2003
 */

public abstract class MethodInvocationOperation implements Operation {

	Operation actor;
	Operation[] parameters;

	List parameterExpressions;

	String signature;

	public MethodInvocationOperation(String signature, Operation actor, Operation[] parameters)
	{
		this.actor = actor;
		this.parameters = parameters;
		this.signature = signature;

		// expressions that need parameters.
		// max-number: actor + all parameters.
		parameterExpressions = new ArrayList(1 + parameters.length);
		if( actor.needsParameters() ){
			parameterExpressions.add(actor);
		}
		for (int i = 0; i < parameters.length; i++) {
			if( parameters[i].needsParameters() ){
				parameterExpressions.add(parameters[i]);
			}
		}

	}

	public boolean needsParameters()
	{
		return !this.parameterExpressions.isEmpty();
	}

	public boolean needsParameter(String name)
	{
		for (int i = 0; i < this.parameterExpressions.size(); i++) {
			if( ((Operation)this.parameterExpressions.get(i)).needsParameter(name) ){
				return true;
			}
		}
		return false;
	}

	public Object execute(Context context) throws OperationException
	{
		Object actorObject = this.actor.execute(context);
		Object[] parameters = evaluateParameters(this.parameters, context);
		return execute(actorObject, parameters);
	}

	public String toString()
	{
		return this.signature;
	}

	protected abstract Object execute(Object actor, Object[] parameters) throws OperationException;

	public static Object[] evaluateParameters(Operation[] parameters, Context context) throws OperationException
	{
		Object[] params = new Object[parameters.length];
		for (int i = 0; i < parameters.length; i++) {
			params[i] = parameters[i].execute(context);
		}
		return params;
	}

}
