package org.jicengine.test;

import junit.framework.TestCase;
import java.awt.Color;
/**
 *
 *
 *
 *
 * @author .timo
 *
 */

public class ColorTest extends TestCase {

	public void testColorFromCdata() throws Exception
	{
		String jic;
		Color result;

		jic = "<color type=\"color\">127,255,0</color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(127,255,0), result);

		jic = "<color type=\"color\">127,255,0,50</color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(127,255,0,50), result);

		jic = "<color type=\"color\">FFFFFF</color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(255,255,255), result);

		jic = "<color class=\"java.awt.Color\" type=\"color\">FF00FF00</color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(255,0,255,0), result);
	}

	public void testTypeExpressions() throws Exception
	{
		String jic;
		Color result;

		jic = "<color type=\"color('127,255,0')\"></color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(127,255,0), result);

		jic = "<color type=\"color('127,255,0',50)\"></color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(127,255,0,50), result);

		jic = "<color type=\"color('FFFFFF')\"></color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(255,255,255), result);

	}

	public void testRgbAndAlphaElements() throws Exception
	{
		String jic;
		Color result;

		jic = "<color type=\"color(rgb)\"><rgb type=\"string\">127,255,0</rgb></color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(127,255,0), result);

		jic = "<color type=\"color(rgb,alpha)\"><rgb type=\"string\">127,255,0</rgb><alpha>50</alpha></color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(127,255,0,50), result);

		jic = "<color type=\"color(rgb)\"><rgb>FFFFFF</rgb></color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(255,255,255), result);

		jic = "<color class=\"java.awt.Color\" type=\"color(rgb,alpha)\" ><rgb>FF00FF</rgb><alpha>0</alpha></color>";
		result = (Color) Utils.buildJicFragment(jic);
		assertSameColor(new java.awt.Color(255,0,255,0), result);

	}

	public void assertSameColor(Color color1, Color color2)
	{
		String msg = "Expected " + color1 + ", was " + color2;
		assertTrue(msg, color1.getRed() == color2.getRed());
		assertTrue(msg, color1.getGreen() == color2.getGreen());
		assertTrue(msg, color1.getBlue() == color2.getBlue());
		assertTrue("Alpha-channel test: " + msg, color1.getAlpha() == color2.getAlpha());
	}
}
