package org.jicengine.test;

import junit.framework.TestCase;
import java.util.*;
/**
 *
 *
 *
 *
 * @author .timo
 *
 */

public class OverridingTest extends TestCase {

	public void testOverridableByAttribute() throws Exception {
		String jic = "<a overridable-by=\"param1\">Default value</a>";
		Utils.jicFragmentEquals(jic, "Default value");
	}

	public void testValueOverridden() throws Exception {
		String jic = "<a overridable-by=\"param1\">Default value</a>";
		Map parameters = new HashMap();
		parameters.put("param1", "param1 value");
		Utils.jicFragmentEquals(jic, parameters, "param1 value");
	}

	public void testOverridableElementWithoutDefaultValue() throws Exception {
		String jic;
		Map parameters;

		jic = "<a overridable-by=\"param1\"/>";
		parameters= new HashMap();
		parameters.put("param1", "param1 value");
		Utils.jicFragmentEquals(jic, parameters, "param1 value");

		// the class-attribute shouldn't affect anything.
		jic = "<a class=\"java.lang.String\" overridable-by=\"param1\"/>";
		parameters= new HashMap();
		parameters.put("param1", "param1 value");
		Utils.jicFragmentEquals(jic, parameters, "param1 value");
	}

	public void testObligatoryOverridingFails() throws Exception {
		String jic;

		jic = "<a overridable-by=\"param1\"/>";
		try {
			Utils.buildJicFragment(jic);
			fail();
		} catch (Exception e){
		}

		// if the class-param is given, the example should work..
		jic = "<a class=\"java.util.Date\" overridable-by=\"param1\"/>";
		Utils.buildJicFragment(jic);
	}

	public void testOverridingObjectOfWrongTypeFails() throws Exception {
		String jic;
		Map parameters;

		jic = "<a class=\"java.util.Date\" overridable-by=\"param1\"/>";
		parameters= new HashMap();
		parameters.put("param1", "a String shouldn't match with a Date.");
		try {
			Utils.buildJicFragment(jic, parameters);
			fail();
		} catch (Exception e){
		}
	}

	public void testActionsInsideOverriddenObjectAreNotExecuted() throws Exception {
		String jic;
		Map parameters;
		Map map = new HashMap();

		jic = "<a type=\"block\" instance=\"map\">";
		jic += "<map id=\"map\" overridable-by=\"map\"/>";
		jic += "<overridable overridable-by=\"param1\">";
		jic +=   "<tryToAlterMap action=\"map.put('key1','value1')\"/>";
		jic += "</overridable>";
		jic += "<not-overridable instance=\"'dummy'\">";
		jic +=   "<tryToAlterMap action=\"map.put('key2','value2')\"/>";
		jic += "</not-overridable>";

		jic += "</a>";

		parameters= new HashMap();
		parameters.put("param1", "param1 value");
		// give the map as a parameter to the JIC-script.
		parameters.put("map", map);

		Utils.buildJicFragment(jic, parameters);

		// if the action inside the <overridable> was skipped, the
		// map shouldn't contain this key:
		assertTrue(!map.containsKey("key1"));

		// map should contain this key:
		assertTrue(map.containsKey("key2"));
	}

	public void testActionParametersAreNotOverridden() throws Exception {
		String jic;
		Map parameters;
		Map map = new HashMap();

		jic = "<a type=\"block\" instance=\"map\">";
		jic += "<map id=\"map\" overridable-by=\"map\"/>";
		jic += "<overridable action=\"map.put('key2',value2)\" overridable-by=\"param1\">";
		jic +=   "<tryToAlterMap action=\"map.put('key1','value1')\"/>";
		jic +=   "<value2>Value 2</value2>";
		jic += "</overridable>";
		jic += "</a>";

		parameters= new HashMap();
		parameters.put("param1", "param1 value");
		// give the map as a parameter to the JIC-script.
		parameters.put("map", map);

		Map expected = new HashMap();
		expected.put("key2","Value 2");

		Utils.jicFragmentEquals(jic, parameters, expected);

		// if the action inside the <overridable> was skipped, the
		// map shouldn't contain this key:
		assertTrue(!map.containsKey("key1"));


	}
}
